//
//  AKMix.h
//  AudioKit
//
//  Created by Aurelius Prochazka on 6/28/12.
//  Copyright (c) 2012 Aurelius Prochazka. All rights reserved.
//

#import "AKAudio.h"
#import "AKParameter+Operation.h"

/** Calculates the weighted mean value of two input signals.
 */

@interface AKMix : AKAudio

/// Create a weighted mean of two inputs using a balance point.
/// @param input1 First input
/// @param input2 Second input
/// @param balancePoint A number from 0 (all input 1) to 1 (all input 2).
- (instancetype)initWithInput1:(AKParameter *)input1
                        input2:(AKParameter *)input2
                        balance:(AKParameter *)balancePoint;

/// Set the minimum balance point.
/// @param minimumBalancePoint The value for which the balance point would indicate all input 1.
- (void)setMinimumBalancePoint:(AKConstant *)minimumBalancePoint;

/// Set the maximum balance point.
/// @param maximumBalancePoint The value for which the balance point would indicate all input 2.
- (void)setMaximumBalancePoint:(AKConstant *)maximumBalancePoint;

@end
